@@ -10,6 +10,7 @@ from prettytable import PrettyTable
 import psutil
 from psutil._common import bytes2human
 
+from jade.common import STATS_DIR
 from jade.events import (
     EVENT_CATEGORY_RESOURCE_UTIL,
     EVENT_NAME_CPU_STATS,
@@ -19,6 +20,7 @@ from jade.events import (
     StructuredLogEvent,
 )
 from jade.loggers import log_event
+from jade.utils.utils import dump_data
 
 
 logger = logging.getLogger(__name__)
@@ -55,13 +57,6 @@ class ResourceMonitor:
         self._update_disk_stats(psutil.disk_io_counters())
         self._update_net_stats(psutil.net_io_counters())
 
-    def log_resource_stats(self):
-        """Logs resource stats information as structured job events."""
-        self.log_cpu_stats()
-        self.log_disk_stats()
-        self.log_memory_stats()
-        self.log_network_stats()
-
     def _update_disk_stats(self, data):
         for stat in self.DISK_STATS:
             setattr(self, stat, getattr(data, stat, 0))
@@ -72,27 +67,14 @@ class ResourceMonitor:
             setattr(self, stat, getattr(data, stat, 0))
         self._last_net_check_time = time.time()
 
-    def log_cpu_stats(self):
-        """Logs CPU resource stats information."""
-        cpu_stats = psutil.cpu_times_percent()._asdict()
-        cpu_stats["cpu_percent"] = psutil.cpu_percent()
+    def get_cpu_stats(self):
+        """Gets CPU current resource stats information."""
+        stats = psutil.cpu_times_percent()._asdict()
+        stats["cpu_percent"] = psutil.cpu_percent()
+        return stats
 
-        log_event(
-            StructuredLogEvent(
-                source=self._name,
-                category=EVENT_CATEGORY_RESOURCE_UTIL,
-                name=EVENT_NAME_CPU_STATS,
-                message="Node CPU stats update",
-                **cpu_stats,
-            )
-        )
-
-    @staticmethod
-    def _mb_per_sec(num_bytes, elapsed_seconds):
-        return float(num_bytes) / ONE_MB / elapsed_seconds
-
-    def log_disk_stats(self):
-        """Logs disk stats."""
+    def get_disk_stats(self):
+        """Gets current disk stats."""
         data = psutil.disk_io_counters()
         stats = {
             "elapsed_seconds": time.time() - self._last_disk_check_time,
@@ -104,32 +86,14 @@ class ResourceMonitor:
         stats["read IOPS"] = float(stats["read_count"]) / stats["elapsed_seconds"]
         stats["write IOPS"] = float(stats["write_count"]) / stats["elapsed_seconds"]
         self._update_disk_stats(data)
+        return stats
 
-        log_event(
-            StructuredLogEvent(
-                source=self._name,
-                category=EVENT_CATEGORY_RESOURCE_UTIL,
-                name=EVENT_NAME_DISK_STATS,
-                message="Node disk stats update",
-                **stats,
-            )
-        )
+    def get_memory_stats(self):
+        """Gets current memory resource stats."""
+        return psutil.virtual_memory()._asdict()
 
-    def log_memory_stats(self):
-        """Logs memory resource stats information."""
-        mem_stats = psutil.virtual_memory()._asdict()
-        log_event(
-            StructuredLogEvent(
-                source=self._name,
-                category=EVENT_CATEGORY_RESOURCE_UTIL,
-                name=EVENT_NAME_MEMORY_STATS,
-                message="Node memory stats update",
-                **mem_stats,
-            )
-        )
-
-    def log_network_stats(self):
-        """Logs memory resource stats information."""
+    def get_network_stats(self):
+        """Gets current network stats."""
         data = psutil.net_io_counters()
         stats = {
             "elapsed_seconds": time.time() - self._last_net_check_time,
@@ -139,10 +103,148 @@ class ResourceMonitor:
         stats["recv MB/s"] = self._mb_per_sec(stats["bytes_recv"], stats["elapsed_seconds"])
         stats["sent MB/s"] = self._mb_per_sec(stats["bytes_sent"], stats["elapsed_seconds"])
         self._update_net_stats(data)
+        return stats
 
+    @staticmethod
+    def _mb_per_sec(num_bytes, elapsed_seconds):
+        return float(num_bytes) / ONE_MB / elapsed_seconds
+
+    @property
+    def name(self):
+        """Return the name of the monitor."""
+        return self._name
+
+
+class ResourceMonitorAggregator:
+    """Aggregates resource utilization stats in memory."""
+
+    def __init__(self, name):
+        self._count = 0
+        self._monitor = ResourceMonitor(name)
+        self._last_stats = self._get_stats()
+        self._summaries = {
+            "average": defaultdict(dict),
+            "maximum": defaultdict(dict),
+            "minimum": defaultdict(dict),
+            "sum": defaultdict(dict),
+        }
+        for resource_type, stat_dict in self._last_stats.items():
+            for stat_name, val in stat_dict.items():
+                self._summaries["average"][resource_type][stat_name] = 0.0
+                self._summaries["maximum"][resource_type][stat_name] = 0.0
+                self._summaries["minimum"][resource_type][stat_name] = sys.maxsize
+                self._summaries["sum"][resource_type][stat_name] = 0.0
+
+    def _get_stats(self):
+        return {
+            CpuStatsViewer.metric(): self._monitor.get_cpu_stats(),
+            DiskStatsViewer.metric(): self._monitor.get_disk_stats(),
+            MemoryStatsViewer.metric(): self._monitor.get_memory_stats(),
+            NetworkStatsViewer.metric(): self._monitor.get_network_stats(),
+        }
+
+    def finalize(self, output_dir):
+        """Finalize the stat summaries and record the results.
+
+        Parameters
+        ----------
+        output_dir : str
+            Directory in which to record the results.
+
+        """
+        for resource_type, stat_dict in self._summaries["sum"].items():
+            for stat_name, val in stat_dict.items():
+                self._summaries["average"][resource_type][stat_name] = val / self._count
+
+        self._summaries.pop("sum")
+        stat_summaries = []
+        for resource_type in (
+            CpuStatsViewer.metric(),
+            DiskStatsViewer.metric(),
+            MemoryStatsViewer.metric(),
+            NetworkStatsViewer.metric(),
+        ):
+            # Make each entry look like what the stat viewers produce.
+            summary = {"batch": self.name, "type": resource_type}
+            for stat_type in self._summaries.keys():
+                summary[stat_type] = self._summaries[stat_type][resource_type]
+            stat_summaries.append(summary)
+
+        path = Path(output_dir) / STATS_DIR
+        filename = path / f"{self.name}_resource_stats.json"
+        dump_data(stat_summaries, filename)
+
+    @property
+    def name(self):
+        """Return the name of the monitor."""
+        return self._monitor.name
+
+    def update_resource_stats(self):
+        """Update resource stats information as structured job events for the current interval."""
+        cur_stats = self._get_stats()
+        for resource_type, stat_dict in self._last_stats.items():
+            for stat_name, val in stat_dict.items():
+                val = cur_stats[resource_type][stat_name]
+                if val > self._summaries["maximum"][resource_type][stat_name]:
+                    self._summaries["maximum"][resource_type][stat_name] = val
+                elif val < self._summaries["minimum"][resource_type][stat_name]:
+                    self._summaries["minimum"][resource_type][stat_name] = val
+                self._summaries["sum"][resource_type][stat_name] += val
+        self._count += 1
+        self._last_stats = cur_stats
+
+
+class ResourceMonitorLogger:
+    """Logs resource utilization stats on periodic basis."""
+
+    def __init__(self, name):
+        self._monitor = ResourceMonitor(name)
+
+    def log_cpu_stats(self):
+        """Logs CPU resource stats information."""
+        cpu_stats = self._monitor.get_cpu_stats()
         log_event(
             StructuredLogEvent(
-                source=self._name,
+                source=self.name,
+                category=EVENT_CATEGORY_RESOURCE_UTIL,
+                name=EVENT_NAME_CPU_STATS,
+                message="Node CPU stats update",
+                **cpu_stats,
+            )
+        )
+
+    def log_disk_stats(self):
+        """Logs disk stats."""
+        stats = self._monitor.get_disk_stats()
+        log_event(
+            StructuredLogEvent(
+                source=self.name,
+                category=EVENT_CATEGORY_RESOURCE_UTIL,
+                name=EVENT_NAME_DISK_STATS,
+                message="Node disk stats update",
+                **stats,
+            )
+        )
+
+    def log_memory_stats(self):
+        """Logs memory resource stats information."""
+        mem_stats = self._monitor.get_memory_stats()
+        log_event(
+            StructuredLogEvent(
+                source=self.name,
+                category=EVENT_CATEGORY_RESOURCE_UTIL,
+                name=EVENT_NAME_MEMORY_STATS,
+                message="Node memory stats update",
+                **mem_stats,
+            )
+        )
+
+    def log_network_stats(self):
+        """Logs memory resource stats information."""
+        stats = self._monitor.get_network_stats()
+        log_event(
+            StructuredLogEvent(
+                source=self.name,
                 category=EVENT_CATEGORY_RESOURCE_UTIL,
                 name=EVENT_NAME_NETWORK_STATS,
                 message="Node net stats update",
@@ -150,6 +252,18 @@ class ResourceMonitor:
             )
         )
 
+    def log_resource_stats(self):
+        """Logs resource stats information as structured job events for the current interval."""
+        self.log_cpu_stats()
+        self.log_disk_stats()
+        self.log_memory_stats()
+        self.log_network_stats()
+
+    @property
+    def name(self):
+        """Return the name of the monitor."""
+        return self._monitor.name
+
 
 class StatsViewerBase(abc.ABC):
     """Base class for viewing statistics"""
@@ -201,7 +315,7 @@ class StatsViewerBase(abc.ABC):
         return self._stat_max_by_batch[batch]
 
     @staticmethod
-    def _get_printable_value(field, val):
+    def get_printable_value(field, val):
         if isinstance(val, float):
             return "{:.3f}".format(val)
         return str(val)
@@ -302,7 +416,7 @@ class StatsViewerBase(abc.ABC):
                 for event in events:
                     row = [event.timestamp]
                     for field, val in event.data.items():
-                        row.append(self._get_printable_value(field, val))
+                        row.append(self.get_printable_value(field, val))
                     table.add_row(row)
                 print(table)
                 print("\n", end="")
@@ -310,15 +424,15 @@ class StatsViewerBase(abc.ABC):
             table.field_names = ["stat"] + list(events[0].data.keys())
             row = ["Average"]
             for field, val in self._calc_batch_averages(batch).items():
-                row.append(self._get_printable_value(field, val))
+                row.append(self.get_printable_value(field, val))
             table.add_row(row)
             row = ["Minimum"]
             for field, val in self._get_batch_minimums(batch).items():
-                row.append(self._get_printable_value(field, val))
+                row.append(self.get_printable_value(field, val))
             table.add_row(row)
             row = ["Maximum"]
             for field, val in self._get_batch_maximums(batch).items():
-                row.append(self._get_printable_value(field, val))
+                row.append(self.get_printable_value(field, val))
             table.add_row(row)
             print(table)
             print("\n", end="")
@@ -326,7 +440,7 @@ class StatsViewerBase(abc.ABC):
         table = PrettyTable(title=f"{self.metric()} averages per interval across batches")
         averages = self._calc_total_averages()
         table.field_names = list(averages.keys())
-        row = [self._get_printable_value(k, v) for k, v in averages.items()]
+        row = [self.get_printable_value(k, v) for k, v in averages.items()]
         table.add_row(row)
         print(table)
         print("\n", end="")
@@ -344,14 +458,14 @@ class StatsViewerBase(abc.ABC):
         for batch, totals in self._stat_sums_by_batch.items():
             row = [batch]
             for stat in stats_to_total:
-                val = self._get_printable_value(stat, totals[stat])
+                val = self.get_printable_value(stat, totals[stat])
                 row.append(val)
             table.add_row(row)
 
         if self._stat_totals:
             total_row = ["total"]
             for stat in stats_to_total:
-                val = self._get_printable_value(stat, self._stat_totals[stat])
+                val = self.get_printable_value(stat, self._stat_totals[stat])
                 total_row.append(val)
             table.add_row(total_row)
             print(table)
@@ -379,7 +493,7 @@ class DiskStatsViewer(StatsViewerBase):
         return "Disk"
 
     @staticmethod
-    def _get_printable_value(field, val):
+    def get_printable_value(field, val):
         if field in ("read_bytes", "write_bytes"):
             val = bytes2human(val)
         elif isinstance(val, float):
@@ -413,7 +527,7 @@ class MemoryStatsViewer(StatsViewerBase):
         return "Memory"
 
     @staticmethod
-    def _get_printable_value(field, val):
+    def get_printable_value(field, val):
         if field == "percent":
             val = "{:.3f}".format(val)
         else:
@@ -432,7 +546,7 @@ class NetworkStatsViewer(StatsViewerBase):
         return "Network"
 
     @staticmethod
-    def _get_printable_value(field, val):
+    def get_printable_value(field, val):
         if field in ("bytes_recv", "bytes_sent"):
             val = bytes2human(val)
         elif isinstance(val, float):
